function proc = dcat(sett, d_train, l_train, d_test, l_test)

% proc = dcat(sett, d_train, l_train, d_test, l_test)
% 
% DCAT (Data Classification and Analysis Tool) is a set of functions that
% can be used to perform several classifications, fetaure extraction or
% score fusion.
% 
% sett      - settings
% d_train   - input training matrix; columns are related to the features,
%             rows to the observations
% l_train   - column vector with numeric labels of the training
%             observations (number of rows must be eaqual to the number of
%             rows in training matrix)
% d_test    - input testing matrix; columns are related to the features,
%             rows to the observations; the matrix must have the same
%             number of columns as the training one (optional)
% l_test    - column vector with numeric labels of the testing
%             observations (number of rows must be eaqual to the number of
%             rows in testing matrix) (optional)
% 
% sett.split_ratio      - ratio of training data in an interval (0,1). If
%                         ratio is an integer >= 1, than there will be
%                         selected only one testing sample on the place
%                         denoted by ratio. To use this option there must
%                         be selected method 'one'.
% sett.split_method     - split method {'random', 'linear', 'one'}
%                         (default: 'random')
% sett.norm             - if set to 1, then the data will be normalized
% sett.rnd              - number of classification rounds
% 
% proc      - structure with the outputs of classification processes

%% Paths and variables
addpath('toolbox');
addpath(['toolbox' filesep 'FEAST' filesep 'FSToolbox']);
addpath(['toolbox' filesep 'DETware']);

if((nargin < 4) || isempty(d_test))
    if(~isfield(sett, 'split_ratio') || isempty(sett.split_ratio))
        sett.split_ratio = 0.75;
    end
    if(~isfield(sett, 'split_method') || isempty(sett.split_method))
        sett.split_method = 'random';
    end
    
    d_test = [];
end
if((nargin < 5) || isempty(l_test))
    l_test = [];
end

%% Run the classification processes
if(~isempty(l_test) || ~isfield(sett, 'rnd'))
    sett.rnd = 1;
elseif(strcmp(sett.split_method, 'one') && isempty(d_test))
    sett.rnd = size(d_train, 1);
end

% Set parallel computing
if(sett.rnd > 1)
    clstr = findResource();
    matlabpool('open', 'local', clstr.ClusterSize);
end

parfor i = 1:sett.rnd
    disp(['Running the classification proces ' num2str(i) ' from ' ...
        num2str(sett.rnd)]);
    
    % Store the temporary data
    r_d_train = d_train;
    r_l_train = l_train;
    r_d_test = d_test;
    r_l_test = l_test;
    
    % Split the data if neccessary
    if(isempty(r_d_test))
        if(strcmp(sett.split_method, 'one'))
            split_ratio = i;
        else
            split_ratio = sett.split_ratio;
        end
        
        disp(['Spliting data to the training and testing part with ' ...
            'ratio: ' num2str(split_ratio) ', method: ' ...
            sett.split_method]);

        [r_d_train, r_d_test, r_l_train, r_l_test] = ...
            split_data(r_d_train, r_l_train, split_ratio, ...
            sett.split_method);
    end

    % Z-normalize the data
    if(isfield(sett, 'norm') && (sett.norm == 1))
        disp('Normalizing the data');

        [r_d_train, r_d_test] = normalize_data(r_d_train, r_d_test);
    end
    
    proc(i) = classification_process(r_d_train, r_l_train, r_d_test, ...
        r_l_test, sett, i);
end

% Free the cores
if(sett.rnd > 1)
    matlabpool close;
end