function proc = classification_process(d_train, l_train, d_test, l_test, sett, rnd)

% proc = classification_process(d_train, l_train, d_test, l_test, sett, rnd)
% 
% This function performs the feature selection according to filtering and
% wrapping methods, classification and score fusion.
% 
% d_train   - input training matrix; columns are related to the features,
%             rows to the observations
% l_train   - column vector with numeric labels of the training
%             observations (number of rows must be eaqual to the number of
%             rows in training matrix)
% d_test    - input testing matrix; columns are related to the features,
%             rows to the observations; the matrix must have the same
%             number of columns as the training one
% l_test    - column vector with numeric labels of the testing
%             observations (number of rows must be eaqual to the number of
%             rows in testing matrix)
% rnd       - the round ID (will be used in console traces)
% 
% sett.labels_descr     - description of labels
% sett.features_descr   - description of features
% sett.cls(i)           - i-th classifier settings
% sett.fus(i)           - i-th score fusion classifier settings
% sett.fsel1_method     - method filtering feature selection (optional)
% sett.fsel1_num_sel    - number of fetaures to be selected during the
%                         filtering method (optional)
% sett.fsel2_method     - method of wrapping feature selection (optional)
% sett.fsel2_max_num    - maximum number of features to be selected during
%                         the wrapping method (optional)
% sett.fsel2_term       - number of cycles without improvement after which
%                         the wrapping method should be terminated
%                         (optional)
% sett.fsel2_rnd        - number of cycles during the wrapping feature
%                         selection
% sett.sel_label        - reference label for the feature matrix (optional)

%% Paths and variables
if((nargin < 6) || isempty(rnd))
    rnd = 'Classification process no. 1 - ';
else
    rnd = ['Classification process no. ' num2str(rnd) ' - '];
end

%% Do the feature selection according to the filtering methods
if(isfield(sett, 'fsel1_method') && isfield(sett, 'fsel1_num_sel'))
    disp([rnd 'selecting ' num2str(sett.fsel1_num_sel) ' fetaures by ' ...
        'filtering method ' upper(sett.fsel1_method)]);
    
    % Select the features
    proc.fsel1_selected = feature_selection_filter(d_train, l_train, ...
        sett.fsel1_num_sel, sett.fsel1_method);

    % Limit the training and testing data only to the selected features
    [d_train, proc.fsel1_features_descr] = limit_data(d_train, ...
        proc.fsel1_selected, sett.features_descr);
    d_test = limit_data(d_test, proc.fsel1_selected);
end

%% Classify and evaluate
for i = 1:length(sett.cls)
    % Store the local data
    d_train2 = d_train;
    d_test2 = d_test;
    
    % Do the second feature selection and limit the data
    if(isfield(sett, 'fsel2_method') && isfield(sett, 'fsel2_max_num') ...
            && isfield(sett, 'fsel2_term') && ...
            isfield(sett, 'fsel2_rnd'))
        disp([rnd 'selecting features by wrapping method ' ...
            upper(sett.fsel2_method)]);
        
        proc.fsel2(i).selected = feature_selection_wrapper(d_train2, ...
            l_train, sett.cls(i), sett.fsel2_method, ...
            sett.fsel2_max_num, sett.fsel2_term, sett.fsel2_rnd);
        
        disp([rnd 'selected ' num2str(length(proc.fsel2(i).selected)) ...
            ' features']);
        
        [d_train2, proc.fsel2(i).features_descr] = limit_data(d_train2, ...
            proc.fsel2(i).selected, proc.fsel1_features_descr);
        d_test2 = limit_data(d_test2, proc.fsel2(i).selected);
    end
    
    % Classify
    disp([rnd 'performing classification by ' upper(sett.cls(i).name)]);
    
    tic;
    [proc.cscore(i).s_train, proc.cscore(i).s_test, ...
        proc.cscore(i).c_train, proc.cscore(i).c_test, ...
        proc.cscore(i).obj] = classifier(d_train2, d_test2, l_train, ...
        sett.cls(i));
    tm = toc;
    
    disp([rnd 'classification time: ' num2str(tm)]);

    % Evaluate
    if(~isempty(l_test))
        proc.evl(i) = evaluation(proc.cscore(i).s_test, ...
            proc.cscore(i).c_test, l_test, sett.labels_descr, ...
            sett.sel_label);
        
        disp([rnd 'overall accuracy: ' num2str(proc.evl(i).acc * 100) ...
            '%']);
    end
end

%% Perform the score fusion
if(isfield(sett, 'fus'))
    for i = 1:length(sett.fus)
        % Compose the training and testing matrices
        d_train3 = [];
        d_test3 = [];

        for j = 1:length(proc.cscore)
            d_train3 = [d_train3 proc.cscore(j).s_train];
            d_test3 = [d_test3 proc.cscore(j).s_test];
        end

        % Classify
        disp([rnd 'performing score fusion by ' upper(sett.fus(i).name)]);

        tic;
        [proc.fscore(i).s_train, proc.fscore(i).s_test, ...
            proc.fscore(i).c_train, proc.fscore(i).c_test, ...
            proc.fscore(i).obj] = classifier(d_train3, d_test3, ...
            l_train, sett.fus(i));
        tm = toc;

        disp([rnd 'classification time: ' num2str(tm)]);

        % Evaluate
        if(~isempty(l_test))
            proc.evlfus(i) = evaluation(proc.fscore(i).s_test, ...
                proc.fscore(i).c_test, l_test, sett.labels_descr, ...
                sett.sel_label);

            disp([rnd 'overall accuracy: ' num2str(proc.evlfus(i).acc * ...
                100) '%']);
        end
    end
end