function [s_train, s_test, c_train, c_test, obj] = classifier(d_train, d_test, l_train, cls)

% [s_train, s_test, c_train, c_test, obj] = classifier(d_train, d_test, l_train, cls)
% 
% This functions performs training and testing using a selected classifier.
% 
% d_train   - input training matrix; columns are related to the features,
%             rows to the observations
% d_test    - input testing matrix; columns are related to the features,
%             rows to the observations; the matrix must have the same
%             number of columns as the training one
% l_train   - column vector with numeric labels of the training
%             observations (number of rows must be eaqual to the number of
%             rows in training matrix)
% cls       - structure with particular classifier setting; these structure
%             must contain at least filed cls.name which determines the
%             selected classifier {'svm', 'bayes', 'da', 'knn', 'tree',
%             'gmm', 'ann'}
% 
% s_train   - column vector which contains the score after the
%             classification of training data
% s_test    - column vector which contains the score after the
%             classification of testing data
% c_train   - the classified training class
% c_test    - the classified testing class
% obj       - object with the trained classifier
% 
% SVM
% cls.kernel = {'linear', 'quadratic', 'polynomial', 'rbf', 'mlp'}
%              (default: 'linear')
% 
% Naive Bayes
% cls.distribution = {'normal', 'kernel', 'mvmn', 'mn'} (default: 'normal')
% cls.prior = {'empirical', 'uniform'} (default: 'uniform')
% 
% Discriminant analysis
% cls.discrimType = {'linear', 'quadratic', 'diagLinear', 'diagQuadratic',
%                   'pseudoLinear', 'pseudoQuadratic'} (default: 'linear')
% cls.prior = {'empirical', 'uniform'} (default: 'uniform')
% 
% k-nearest neighbors
% cls.K = the number of nearest neighbors (default: 1)
% cls.Distance = {'euclidean', 'seuclidean', 'cityblock', 'chebychev',
%                'minkowski', 'mahalanobis', 'cosine', 'correlation',
%                'spearman', 'hamming', 'jaccard'} (default: 'euclidean')
% 
% Classification trees
% cls.SplitCriterion = {'gdi', 'twoing', 'deviance'} (default: 'gdi')
% cls.prior = {'empirical', 'uniform'} (default: 'uniform')
% 
% GMM
% cls.k = number of mixtures (default: 1)

%% Paths and variables
if(~isfield(cls, 'name'))
    error(['The input structure ''cls'' must contain field ''name'' ' ...
        'which determines the selected classifier.']);
end

%% Train and classify
switch(cls.name)
    case 'svm'
        if(~isfield(cls, 'kernel'))
            cls.kernel = 'linear';
        end
        
        obj = svmtrain(d_train, l_train, 'kernel_function', ...
            cls.kernel);
        
        s_train = svmclassify(obj, d_train);
        s_test = svmclassify(obj, d_test);
        
        c_train = s_train;
        c_test = s_test;
    case 'bayes'
        if(~isfield(cls, 'distribution'))
            cls.distribution = 'normal';
        end
        if(~isfield(cls, 'prior'))
            cls.prior = 'uniform';
        end
        
        obj = NaiveBayes.fit(d_train, l_train, 'Distribution', ...
            cls.distribution, 'Prior', cls.prior);
        
        s_train = predict(obj, d_train);
        s_test = predict(obj, d_test);
        
        c_train = s_train;
        c_test = s_test;
    case 'da'
        if(~isfield(cls, 'discrimType'))
            cls.discrimType = 'linear';
        end
        if(~isfield(cls, 'prior'))
            cls.prior = 'uniform';
        end
        
        obj = ClassificationDiscriminant.fit(d_train, l_train, ...
            'discrimType', cls.discrimType, 'Prior', cls.prior);
        
        s_train = predict(obj, d_train);
        s_test = predict(obj, d_test);
        
        c_train = s_train;
        c_test = s_test;
    case 'knn'
        if(~isfield(cls, 'K'))
            cls.K = 1;
        end
        if(~isfield(cls, 'Distance'))
            cls.Distance = 'euclidean';
        end
        
        label = knnsearch(d_train,d_train, 'K', cls.K, 'Distance', ...
            cls.Distance);
        s_train = l_train(mode(label,2));
        
        label = knnsearch(d_train,d_test, 'K', cls.K, 'Distance', ...
            cls.Distance);
        s_test = l_train(mode(label,2));
        
        c_train = s_train;
        c_test = s_test;
        
        obj = [];
    case 'tree'
        if(~isfield(cls, 'SplitCriterion'))
            cls.SplitCriterion = 'gdi';
        end
        if(~isfield(cls, 'prior'))
            cls.prior = 'uniform';
        end
        
        obj = ClassificationTree.fit(d_train, l_train, 'Prior', ...
            cls.prior, 'SplitCriterion', cls.SplitCriterion);
        
        s_train = predict(obj, d_train);
        s_test = predict(obj, d_test);
        
        c_train = s_train;
        c_test = s_test;
    case 'gmm'
        if(~isfield(cls, 'k'))
            cls.k = 1;
        end
        
        obj = GMMfit(d_train, l_train, cls.k);
        
        s_train = GMMclassify(obj, d_train);
        s_test = GMMclassify(obj, d_test);
        
        c_train = s_train;
        c_test = s_test;
    case 'ann'
        labels = unique(l_train);

        T = zeros(length(labels), size(d_train,1));

        for i = 1:length(labels)
            T(i,l_train == labels(i)) = 1;
        end

        obj = patternnet(length(labels)*3);
        obj.trainParam.showWindow = false;
        obj = train(obj, d_train.', T);
        
        score = sim(obj, d_train.');
        [tmp, ind] = max(score);        
        s_train = labels(ind);

        score = sim(obj, d_test.');
        [tmp, ind] = max(score);        
        s_test = labels(ind);
        
        c_train = s_train;
        c_test = s_test;
    otherwise
        error(['Classifier ' cls.name ' is not supported.']);
end