function evl = evaluation(score, l_pred, l_true, labels_descr, sel_label)

% evl = evaluation(score, l_pred, l_true, labels_descr, sel_label)
% 
% This function evaluates the classifier using confusion matrix, overal
% ccuracy, particular accuracies, sensitivity, specificity, equal error
% rate, minimum of detection cost function and area under ROC curve.
% 
% score         - column vector with classifier scores
% l_pred        - column vector with with predicted classes' IDs
% l_true        - column vector with with true classes' IDs
% labels_descr  - description of labels
% sel_label     - this varibale contains the name of label that should be
%                 taken as reference during the calculation of sensitivity,
%                 specificity, EER, DCF and AUC (optional variable)
% 
% evl.acc           - overall accuracy
% evl.cf_mtx        - confusion matrix
% evl.acc_table     - table of accuracies for particular classes
% evl.sensitivity   - sensitivity
% evl.specificity   - specificity
% evl.s_true        - scores of true cases
% evl.s_false       - scores of false cases
% evl.eer           - equal error rate
% evl.DCF_min       - minimum of detection cost function
% evl.DCF_miss      - P(miss) of DCF
% evl.DCF_fail      - P(fail) of DCF
% evl.AUC           - area under ROC curve

%% Extract the labels
if(exist('labels_descr', 'var'))
    labels = (1:length(labels_descr)).';
else
    labels = unique([l_pred; l_true]);
end

% Switch the labels if neccessary
if((length(labels) <= 2) && (nargin > 4))    
    if(~any(strcmp(labels_descr, sel_label)))
        error(['Label ' sel_label ' is not present in the data.']);
    end

    if(find(strcmp(labels_descr, sel_label)) == 2)
        labels = flipud(labels);
    end
elseif(nargin > 4)
    error(['Cannot calculate sensitivity and specificity for ' ...
            'more than 2 classes.']);
end

%% Calculate the overal accuracy
evl.acc = sum(l_pred == l_true)/length(l_pred);


if(exist('labels_descr', 'var'))
    % Compose the confusion matrix
    tmp = zeros(length(labels));

    for i = 1:length(labels)
        for j = 1:length(labels)
            tmp(i,j) = sum(l_pred(l_true == labels(i)) == labels(j));
        end
    end

    cf_mtx = cell(length(labels)+1);
    cf_mtx(1,2:end) = labels_descr(labels);
    cf_mtx(2:end,1) = labels_descr(labels);
    cf_mtx(2:end,2:end) = num2cell(tmp);

    evl.cf_mtx = cf_mtx;

    % Calculate the particular accuracies
    acc_table = cell(2,size(evl.cf_mtx,1)-1);
    acc_table(1,:) = evl.cf_mtx(1,2:end);

    for i = 1:size(acc_table,2)
        acc_table{2,i} = evl.cf_mtx{i+1,i+1}/ ...
            sum(cell2mat(evl.cf_mtx(i+1,2:end)));
    end

    evl.acc_table = acc_table;

    % Calculate sensitivity, specificity, EER, DCF and AUC
    if(length(labels) == 2)
        evl.sensitivity = evl.acc_table{2,1};
        evl.specificity = evl.acc_table{2,2};

        % Change the score values if necessary
        if(~any(labels - round(labels)) && (labels(1) < labels(2)))
            sel = l_pred == labels(1);

            score(sel) = 1;
            score(~sel) = 0;
        end

        evl.s_true = score(l_true == labels(1));
        evl.s_false = score(l_true == labels(2));

        % Compute Pmiss and Pfa from experimental detection output scores
        [P_miss, P_fa] = Compute_DET(evl.s_true, evl.s_false);

        % Calculate EER
        evl.eer = P_miss(find(P_miss >= P_fa, 1, 'first'));

        % Find the minimum of DCF
        Set_DCF(1, 1, 0.5);
        [evl.DCF_min evl.DCF_miss evl.DCF_fail] = Min_DCF(P_miss, P_fa);

        % Calculate the AUC
        evl.AUC = 1-sum(abs(diff(P_miss)).*P_fa(1:end-1));
    end
end