function evaluation_overall(proc, sett)

% evaluation_overall(proc, sett)
% 
% This function performs the overall evaluation over all classifiers and
% rounds of validation.
% 
% proc  - structure with the outputs of classification processes
% sett  - settings

%% Paths and variables
res.acc = [];
res.sensitivity = [];
res.specificity = [];
res.eer = [];
res.DCF_min = [];
res.DCF_miss = [];
res.DCF_fail = [];
res.AUC = [];

%% Feed the structure by the results
n_cls = length(sett.cls);
if(isfield(sett, 'fus'))
    n_fus = length(sett.fus);
else
    n_fus = 0;
end

names = fieldnames(res);

for i = 1:length(proc) % Over all processes
    for j = 1:length(names) % Over all parameters
        if(isfield(proc(1).evl(1), names{j}))
            tmp = [];
            
            if(n_fus)
                tmp = proc(i).evlfus.(names{j});
            end
            
            res.(names{j}) = [res.(names{j}); proc(i).evl.(names{j}) tmp];
        end
    end
end

%% Prepare the output table
results = cell(n_cls + n_fus + 1, 0);

for i = 1:n_cls
    if(isfield(sett.cls(i), 'descr') && ~isempty(sett.cls(i).descr))
        results{i + 1,1} = sett.cls(i).descr;
    else
        results{i + 1,1} = upper(sett.cls(i).name);
    end
end

for i = 1:n_fus
    results{n_cls + i + 1,1} = ['f' upper(sett.fus(i).name)];
end

for i = 1:length(names)
    if(~isempty(res.(names{i})))        
        if(length(proc) > 1)
            results(:,end+1) = [[lower(names{i}) '(mean)']; ...
                num2cell(nanmean(res.(names{i})).')];
            results(:,end+1) = [[lower(names{i}) '(std)']; ...
                num2cell(nanstd(res.(names{i})).')];
        else
            results(:,end+1) = [lower(names{i}); ...
                num2cell(res.(names{i}).')];
        end
    end
end

%% Prepare the statistics from the selected features
selected = [];
selected_all = cell(0,0);

for i = 1:n_cls
    selected_tmp = cell(0,0);
    feat(i).cls = upper(sett.cls(i).name);
    
    % Get the selected features
    for j = 1:length(proc)
        if(isfield(proc(j), 'fsel2'))
            selected_tmp = [selected_tmp; ...
                proc(j).fsel2(i).features_descr];
            
            selected_all = [selected_all; proc(j).fsel2(i).features_descr];
        elseif(isfield(proc(j), 'fsel1_features_descr'))
            selected_tmp = [selected_tmp; ...
                proc(j).fsel1_features_descr];
            
            selected_all = [selected_all; proc(j).fsel1_features_descr];
        end
    end
    
    % Get the statistics
    if(~isempty(selected_tmp))
        [feat(i).stat,tmp,ind] = unique(selected_tmp);
        feat(i).stat = [feat(i).stat num2cell(hist(ind, max(ind)).')];
        
        [tmp, ind] = sort(cell2mat(feat(i).stat(:,2)), 'descend');
        feat(i).stat = feat(i).stat(ind,:);
    end
end

% Store the statistics to the table
if(~isempty(selected_all))
    % Get the overall statistics
    [selected_all,tmp,ind] = unique(selected_all);
    selected_all = [selected_all num2cell(hist(ind, max(ind)).')];
    [tmp, ind] = sort(cell2mat(selected_all(:,2)), 'descend');
    selected_all = selected_all(ind,:);
    
    % Store data to the table
    selected = cell(size(selected_all,1)+1, n_cls*2);
    
    for i = 1:n_cls
        selected{1,(i-1)*2+1} = feat(i).cls;
        selected{1,(i-1)*2+2} = '#';
        
        selected(2:size(feat(i).stat,1)+1, (i-1)*2+1:(i-1)*2+2) = ...
            feat(i).stat;
    end
    
    if(n_cls > 1)
        selected(:,end+1:end+2) = [{'OVERALL' '#'}; selected_all];
    end
end

%% Store the tables
[pathstr, name] = fileparts(sett.out);

pth_xlsx = fullfile(pathstr, [name '_scores.xlsx']);
pth_mat = fullfile(pathstr, [name '_process.mat']);

disp(['Storing table with the results to ' pth_xlsx]);
if(exist(pth_xlsx, 'file'))
    delete(pth_xlsx);
end

xlswrite(pth_xlsx, results);

disp(['Storing classification process results to ' pth_mat]);
save(pth_mat, 'proc', 'sett');

if(~isempty(selected))
    pth_xlsx = fullfile(pathstr, [name '_selected.xlsx']);
    
    disp(['Storing table with the selected features to ' pth_xlsx]);
    if(exist(pth_xlsx, 'file'))
        delete(pth_xlsx);
    end

    xlswrite(pth_xlsx, selected);
end