function selected = feature_selection_wrapper(data, labels, cls, method, max_num, term, rnd)

% selected = feature_selection_wrapper(data, labels, cls, method, max_num, term, rnd)
% 
% This function selects the significant features according to the wrapper
% or filtering methods methods extended by the classification. There are
% supported different methods (computational burden is in an ascending
% order): Conditional Mutual Info Maximisation, Min-Redundancy
% Max-Relevance, Joint Mutual Information, Double Input Symmetrical
% Relevance, Conditional Redundancy, Sequential Floating Forward Selection.
% 
% data      - input data matrix; columns are related to the features, rows
%             to the observations
% labels    - column vector with numeric labels of the observations (number
%             of rows must be eaqual to the number of rows in data)
% cls       - structure with particular classifier setting; these structure
%             must contain at least filed cls.name which determines the
%             selected classifier
% method    - feature selection method {'cmim', 'mrmr', 'cmim', 'disr',
%             'condred', 'sffs'} (default: 'sffs')
% max_num   - maximum number of features to be selected (default: number of
%             features in data matrix)
% term      - number of cycles without improvement after which the
%             selection process will be terminated (default: 10)
% rnd       - number of validation rounds (default: 10)
% 
% selected  - column vector with selected IDs
% 
% Implemented according to
% Brown, G. and Pocock, A. and Zhao, M.J. and Luj{\'a}n, M.. Conditional
% Likelihood Maximisation: A Unifying Framework for Information Theoretic
% Feature Selection . In Journal of Machine Learning Research, (13) 27--66,
% Year 2012.

%% Paths and variables
if((nargin < 4) || isempty(method))
    method = 'sffs';
end
if((nargin < 5) || isempty(max_num))
    max_num = size(data,2);
end
if((nargin < 6) || isempty(term))
    term = 10;
end
if((nargin < 7) || isempty(rnd))
    rnd = 10;
end

if(~iscolumn(labels))
    error('Input variable ''labels'' must be a column vector.');
end

if(isrow(data))
    error(['Input variable ''data'' is a row vector. There must be ' ...
        'more lines to be able to split the matrix.']);
end

if(length(labels) ~= size(data,1))
    error(['Variables ''data'' and ''labels'' must have the same ' ...
        'number of rows.']);
end

if(size(data,2) < max_num)
    warning(['Number of features to be selected is ' num2str(max_num) ...
        '. Number of features in data matrix is ' num2str(size(data,2)) ...
        '. The variable ''max_num'' will be set to ' ...
        num2str(size(data,2)) '.']);
    
    max_num = num2str(size(data,2));
end

if(max_num < 1)
    error('Variable ''max_num'' must be greather than 0.');
end

if(term > size(data,2))
    term = size(data,2);
end

%% Select the features
selected = [];
accuracy = -1;

for i = 1:rnd
    % Split the data
    [d_train, d_test, l_train, l_test] = split_data(data, labels, 0.75, ...
        'random');

    % Prepare the accuracy array
    acc = -(1:term+1).';

    if(strcmp(method, 'sffs'))
        sel_loc = [];

        acc_tmp = -1;
        sel_tmp = [];

        cnt = 1;
        stp = sum(1:size(data,2))+1;

        while(cnt < stp)
            ind = mod(cnt, size(data,2));        
            if(ind == 0)
                ind = size(data,2);
            end

            cnt = cnt + 1;

            % Check if the fetaure is not already selected
            if(any(sel_loc == ind))
                continue;
            end

            % Get the selected IDs
            tmp = [sel_tmp; ind];

            % Limit the training and tetsing data
            d_train_l = limit_data(d_train, tmp);
            d_test_l = limit_data(d_test, tmp);

            try
                % Classify
                [s_train, s_test, c_train, c_test] = ...
                    classifier(d_train_l, d_test_l, l_train, cls);

                % Evaluate
                evl = evaluation(s_test, c_test, l_test);

                % Update the variables    
                if(evl.acc > acc_tmp)
                    acc_tmp = evl.acc;

                    if(evl.acc > max(acc))
                        sel_loc = tmp;
                    end
                end
            catch
            end

            % Updates after the end of set
            if(ind == size(data,2))
                disp(['Wrapping method SFFS (' num2str(i) '/' ...
                    num2str(rnd) '): level ' num2str(length(tmp)) ...
                    ', iteration ' num2str(cnt-1) ...
                    ' - classification by ' upper(cls.name)]);

                acc = circshift(acc,1);
                acc(1) = acc_tmp;

                acc_tmp = -1;
                sel_tmp = sel_loc;

                if(acc(end) >= max(acc(1:end-1)))
                    break;
                end
                if(length(sel_loc) == max_num)
                    break;
                end
            end
        end
    else
        disp(['Wrapping method ' upper(method) ' (' num2str(i) '/' ...
            num2str(rnd) ') - classification by ' upper(cls.name)]);
        
        % Select the fetaures using the filtering method
        tmp = feature_selection_filter(d_train, l_train, max_num, method);

        for j = 1:max_num
            % Limit the training and tetsing data
            d_train_l = limit_data(d_train, tmp(1:j));
            d_test_l = limit_data(d_test, tmp(1:j));

            try
                % Classify
                [s_train, s_test, c_train, c_test] = ...
                    classifier(d_train_l, d_test_l, l_train, cls);

                % Evaluate
                evl = evaluation(s_test, c_test, l_test);

                % Update and terminate if neccesary        
                acc = circshift(acc,1);
                acc(1) = evl.acc;
            catch
                % Update and terminate if neccesary        
                acc = circshift(acc,1);
                acc(1) = -inf;
            end

            if(acc(1) > max(acc(2:end)))
                sel_loc = tmp(1:j);
            elseif(acc(end) >= max(acc(1:end-1)))
                break;
            end
        end
    end
    
    % Update the selected IDs
    if(max(acc) > accuracy)
        selected = sel_loc;
        accuracy = max(acc);
    end
end