function [d_train, d_test, l_train, l_test] = split_data(data, labels, ratio, method)

% [d_train, d_test, l_train, l_test] = split_data(data, labels, ratio, method)
% 
% This function splits the inpud data matrix into training and testing data
% according to the selected ratio.
% 
% data      - input data matrix; columns are related to the features, rows
%             to the observations
% labels    - column vector with numeric labels of the observations (number
%             of rows must be eaqual to the number of rows in data)
% ratio     - ratio of training data in an interval (0,1). If ratio is an
%             integer >= 1, than there will be selected only one testing
%             sample on the place denoted by ratio. To use this option
%             there must be selected method 'one'.
% method    - selection method {'random', 'linear', 'one'} (default:
%             'random')
% 
% d_train   - matrix with training data
% d_test    - matrix with tetsing data
% l_train   - labels of training data
% l_test    - labels of testing data

%% Paths and variables
if((nargin < 4) || isempty(method))
    method = 'linear';
end

%% Check the input variables
if(~iscolumn(labels))
    error('Input variable ''labels'' must be a column vector.');
end

if(isrow(data))
    error(['Input variable ''data'' is a row vector. There must be ' ...
        'more lines to be able to split the matrix.']);
end

if(((ratio <= 0) || (ratio > 1)) && ~strcmp(method, 'one'))
    error(['Invalid value of variable ''ratio'' = ' num2str(ratio) ...
        '. The value must be in interval (0;1).']);
end

if(length(labels) ~= size(data,1))
    error(['Variables ''data'' and ''labels'' must have the same ' ...
        'number of rows.']);
end

%% Split the data into training and testing subsets
if(strcmp(method, 'one'))
    if(ratio > size(data,1))
        error(['Variable ''ratio'' in method ''one'' is higher than ' ...
            'the number of rows of data matrix.']);
    end
    
    sel_test = zeros(length(labels),1);
    sel_test(ratio) = 1;
    
    sel_test = sel_test == 1;
    sel_train = ~sel_test;
else
    [n, id] = hist(labels, unique(labels));

    % Check the number of training/testing samples in each class
    n_train = fix(n*ratio);
    n_test = n - n_train;

    if(any(n_train < 1))
        ind = find(n_train < 1, 1, 'first');

        error(['There were selected 0 training samples in class ' ...
            num2str(id(ind)) '. Number of samples in this class is ' ...
            num2str(n(ind)) ', training ratio = ' num2str(ratio) ...
            '. Increase the ratio or the number of samples in class.']);
    end

    if(any(n_test < 1))
        ind = find(n_test < 1, 1, 'first');

        error(['There were selected 0 testing samples in class ' ...
            num2str(id(ind)) '. Number of samples in this class is ' ...
            num2str(n(ind)) ', 1-ratio = ' num2str(1-ratio) ...
            '. Decrease the ratio or increase the number of samples ' ...
            'in class.']);
    end

    % Split the data
    rows = 1:length(labels);
    sel_train = zeros(length(labels),1);

    switch(method)
        case 'random'
            for i = 1:length(n)
                sel = datasample(rows(labels == id(i)),n_train(i), ...
                    'Replace',false);
                sel_train(sel) = 1;
            end
        case 'linear'
            for i = 1:length(n)
                sel = rows(labels == id(i));
                sel_train(sel(1:n_train(i))) = 1;
            end
        otherwise
            error(['Unknow method of data split. Method ''' method ...
                ''' is not supported.']);
    end
    
    sel_train = sel_train == 1;
    sel_test = ~sel_train;
end

d_train = data(sel_train,:);
d_test = data(sel_test,:);

l_train = labels(sel_train,:);
l_test = labels(sel_test,:);